@@ -23,7 +23,6 @@
 
 #include <QList>
 #include <QAction>
-#include <QFileInfo>
 #include <QWidget>
 #include <QVariantList>
 #include <QUrl>
@@ -32,6 +31,7 @@
 #include <QInputDialog>
 
 #include <KPluginFactory>
+#include <KPluginLoader>
 #include <KLocalizedString>
 
 K_PLUGIN_FACTORY_WITH_JSON(TagsFileItemActionFactory, "tagsfileitemaction.json", registerPlugin<TagsFileItemAction>();)
@@ -44,47 +44,44 @@ TagsFileItemAction::TagsFileItemAction(QObject* parent, const QVariantList&)
     m_menu->setIcon(QIcon::fromTheme(QStringLiteral("tag")));
 
     connect(&m_tagsLister, &KCoreDirLister::itemsAdded, this, [this](const QUrl&, const KFileItemList& items) {
-        const QStringList fileTags = m_metaData->tags();
 
-        // The file may be located outside an indexed path, or is not indexed yet
-        // Show the complete tag list, i.e. the union of file and index DB tags
-        QStringList allTags;
-        allTags.reserve(fileTags.size() + items.size());
-        allTags.append(fileTags);
         for (const KFileItem &item: items) {
-            allTags.append(item.name());
-        }
-        allTags.sort(Qt::CaseInsensitive);
-        allTags.removeDuplicates();
+            const QString name = item.name();
 
-        for (const QString name : qAsConst(allTags)) {
             QAction* action = m_menu->addAction(QIcon::fromTheme(QStringLiteral("tag")), name);
             action->setCheckable(true);
-            action->setChecked(fileTags.contains(name));
+            action->setChecked(m_metaData->tags().contains(name));
 
-            connect(action, &QAction::triggered, this, [this, name](bool isChecked) {
-                QStringList newTags = m_metaData->tags();
-                if (isChecked) {
-                    newTags.append(name);
+            connect(action, &QAction::triggered, this, [this, action] {
+                if (action->isChecked()) {
+                    QStringList newTags = m_metaData->tags();
+                    // HACK the first character of QAction::text is '&'
+                    newTags.append(action->text().remove(0,1));
+                    m_metaData->setTags(newTags);
                 } else {
-                    newTags.removeAll(name);
+                    QStringList newTags = m_metaData->tags();
+                    // HACK the first character of QAction::text is '&'
+                    newTags.removeAll(action->text().remove(0,1));
+                    m_metaData->setTags(newTags);
                 }
-                m_metaData->setTags(newTags);
             });
         }
     });
 
-    newAction = new QAction(i18n("Create New..."));
+    QAction* newAction = new QAction(i18n("Create New..."));
     newAction->setIcon(QIcon::fromTheme(QStringLiteral("tag-new")));
 
     connect(newAction, &QAction::triggered, this, [this] {
         QString newTag = QInputDialog::getText(m_menu, i18n("New tag"), i18n("New tag:"), QLineEdit::Normal);
         QStringList tags = m_metaData->tags();
-        if (!tags.contains(newTag)) {
-            tags.append(newTag);
-            m_metaData->setTags(tags);
-        }
+        tags.append(newTag);
+        m_metaData->setTags(tags);
     });
+
+    m_menu->addAction(newAction);
+    m_menu->addSeparator();
+
+    m_tagsLister.openUrl(QUrl("tags:/"), KCoreDirLister::OpenUrlFlag::Reload);
 }
 
 TagsFileItemAction::~TagsFileItemAction()
@@ -94,27 +91,13 @@ TagsFileItemAction::~TagsFileItemAction()
 
 QList<QAction*> TagsFileItemAction::actions(const KFileItemListProperties& fileItemInfos, QWidget* parentWidget)
 {
+    Q_UNUSED(parentWidget);
 
     if (fileItemInfos.urlList().size() > 1) {
         return {};
     }
 
-    QString filePath = fileItemInfos.urlList()[0].toLocalFile();
-    if (!QFileInfo(filePath).isWritable()) {
-        return {};
-    }
-
-    m_metaData = new KFileMetaData::UserMetaData(filePath);
-    if (!m_metaData->isSupported()) {
-        return {};
-    }
-
-    m_tagsLister.openUrl(QUrl(QStringLiteral("tags:/")), KCoreDirLister::OpenUrlFlag::Reload);
-
-    m_menu->clear();
-    m_menu->addAction(newAction);
-    m_menu->addSeparator();
-    m_menu->setParent(parentWidget, Qt::Popup);
+    m_metaData = new KFileMetaData::UserMetaData(fileItemInfos.urlList()[0].toLocalFile());
 
     return {m_menu->menuAction()};
 }
